from typing import List, Any, Type

from pydantic import BaseModel, create_model


class CatwalkDataType(BaseModel):
    key: str
    description: str
    example: dict
    json_schema: dict = None
    src: str
    main: str


class CatwalkEvaluation(BaseModel):
    key: str
    description: str
    example: dict
    json_schema: dict = None
    src: str
    main: str


class BaseDateType(BaseModel):
    name: str


class BaseEvaluation(BaseModel):
    name: str
    question: str


class CatwalkPlugin:
    name: str
    data_types: List[CatwalkDataType]
    evaluations: List[CatwalkEvaluation]

    def __init__(self, name: str):
        self.name = name
        self.data_types = []
        self.evaluations = []

    def declare_data_type(
        self,
        key: str,
        *,
        description: str,
        example: dict,
        src: str,
        main: str,
        model: Type[BaseModel] = None,
    ):
        data_model = type(
            f"DataType{key}", (BaseDateType, model) if model else (BaseDateType,), {}
        )
        self.data_types.append(
            CatwalkDataType(
                key=key,
                description=description,
                example=example,
                src=src,
                main=main,
                json_schema=data_model.schema(),
            )
        )
        return self

    def declare_evaluation(
        self,
        key: str,
        *,
        description: str,
        example: dict,
        src: str,
        main: str,
        model: Type[BaseModel] = None,
    ):
        data_model = type(
            f"Evaluation{key}",
            (BaseEvaluation, model) if model else (BaseEvaluation,),
            {},
        )
        self.evaluations.append(
            CatwalkEvaluation(
                key=key,
                description=description,
                example=example,
                src=src,
                main=main,
                json_schema=data_model.schema(),
            )
        )
        return self

    def dict(self):
        return {
            "name": self.name,
            "data_types": [x.dict() for x in self.data_types],
            "evaluations": [x.dict() for x in self.evaluations],
        }



from typing import List, Any, Union, Optional, Dict

from pydantic import BaseModel, Field


class Type(BaseModel):
    name: str

    class Config:
        extra = "allow"


class Evaluation(BaseModel):
    question: str
    name: str

    class Config:
        extra = "allow"


class Query(BaseModel):
    name: str
    value: Any
    type: Union[Type, str]


class Response(BaseModel):
    name: str
    value: Any
    type: Union[Type, str]
    evaluation: List[Evaluation]


class Context(BaseModel):
    name: str
    value: Any
    type: Union[Type, str]


class Submitter(BaseModel):
    name: str
    version: str


class CommonCaseFormat(BaseModel):
    case_id: str = Field(
        None,
        description="Case id given by the user used for identifying specific cases.",
    )
    version: int = Field(1, description="Version of the Common Case Format.")
    submitter: Submitter = Field(..., description="Who is the submitter of the case")
    query: List[Query] = Field(
        ...,
        description="List of query objects that describes what was the input for the case",
    )
    response: List[Response] = Field(
        ..., description="List of response objects that describes what was the response"
    )
    context: Optional[List[Context]] = Field(
        None, description="Context that was available for model"
    )
    metadata: Dict[str, Any] = Field(
        ..., description="Additional metadata that will be saved with the case"
    )


class OpenCase(CommonCaseFormat):
    id: str
    created_at: str
    is_archived: Optional[bool] = None
    is_open: Optional[bool] = None
